/* ----------------------------------------------------- */
/* Battery Meter                                         */
/* Dave MacLeod <davmac@netcomuk.co.uk>                  */
/*                                                       */
/* ----------------------------------------------------- */
/* v0.1     15/04/97    Creation                         */
/*                                                       */
/* ----------------------------------------------------- */
/* v0.2     04/06/97    fix number formatting bug        */
/*                                                       */
/*                      only "do the do" if the voltage  */
/*                      has changed since last time      */
/*                                                       */
/* ----------------------------------------------------- */
 
#pragma pack(2)
#include <System/SysAll.h>
#include <UI/UIAll.h>

#define MainForm  1000
#define AboutBat  2000
#define MenuAbout 3001
#define MenuSave  3002
#define MenuExp   3003

/* ----------------------------------------------------- */
/* Globals                                               */
/* ----------------------------------------------------- */

static UInt GVolt,
            GWarn,
            GCrit;
static SysBatteryKind GType;

/* ----------------------------------------------------- */
/* GetObjPtr                                             */
/*   This function returns the pointer to a specified    */
/*   object                                              */
/* Parms                                                 */
/*   The object identifier (as specified in the .rcp file*/
/* Returns                                               */
/*   A pointer to the object                             */
/* ----------------------------------------------------- */

static VoidPtr GetObjPtr(Int objID)
{
  FormPtr frm;
  frm = FrmGetActiveForm();
  return(FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, objID)));
}

/* ----------------------------------------------------- */
/* Voltage                                               */
/*   This function returns a formatted text string,      */
/*   in the format 9.99v                                 */
/* Parms                                                 */
/*   The voltage as a number of thousandths of a volt    */
/* Returns                                               */
/*   A pointer to the text string containing the         */
/*   formatted voltage.                                  */
/* ----------------------------------------------------- */

void Voltage(int Volts,Ptr RetnStr)
{
  char wrk[10];

  StrIToA(wrk,Volts/100);
  StrCopy(RetnStr,wrk);
  StrCat(RetnStr,".");
  StrIToA(wrk,(Volts/10)-((Volts/100)*10));
  StrCat(RetnStr,wrk);
  StrIToA(wrk,Volts-((Volts/10)*10));
  StrCat(RetnStr,wrk);
  StrCat(RetnStr,"v");
}

/* ----------------------------------------------------- */
/* MainViewHandleEvent                                   */
/*   This function loads a form and set the event        */
/*   handler for the form.                               */
/* Parms                                                 */
/*   event - Pointer to an event type structure          */
/* Returns                                               */
/*   True - if the event has been dealt with             */
/* ----------------------------------------------------- */

static Boolean MainViewHandleEvent(EventPtr event)
{
  char temptxt[10];
  UInt oldvolt, volt, warnlev, critlev;
  SysBatteryKind btype;
  Boolean plugged;
  Boolean handled = false;
  
  switch (event->eType)
  {
    case frmOpenEvent:
      FrmDrawForm(FrmGetActiveForm());
      handled = true;
      break;
    case nilEvent:
      volt = SysBatteryInfo(false,&warnlev,&critlev,NULL,&btype,NULL);
      if (volt!=GVolt)
      {
        GVolt=volt;
        GWarn=warnlev;
        GCrit=critlev;
        GType=btype;

        Voltage(volt,temptxt);
        WinDrawChars(temptxt,StrLen(temptxt),100,20);
        Voltage(warnlev,temptxt);
        WinDrawChars(temptxt,StrLen(temptxt),100,40);
        Voltage(critlev,temptxt);
        WinDrawChars(temptxt,StrLen(temptxt),100,60);
        if (btype==sysBatteryKindAlkaline)
          StrCopy(temptxt,"Alkaline");
        else
          if (btype==sysBatteryKindNiCad)
            StrCopy(temptxt,"NiCad");
          else
            if (btype==sysBatteryKindLithium)
              StrCopy(temptxt,"Lithium");
            else
              StrCopy(temptxt,"Unknown");
          WinDrawChars(temptxt,StrLen(temptxt),100,80);
      }
      handled = true;
      break;
  }
  return(handled);
}

/* ----------------------------------------------------- */
/* ApplicationHandleEvent                                */
/*   This function loads a form and set the event        */
/*   handler for the form.                               */
/* Parms                                                 */
/*   event - Pointer to an event type structure          */
/* Returns                                               */
/*   True - if the event has been dealt with             */
/* ----------------------------------------------------- */

static Boolean ApplicationHandleEvent(EventPtr event)
{
  FormPtr frm;
  Int formId;
  Boolean handled = false;

  if (event->eType == frmLoadEvent)
  {
    formId = event->data.frmLoad.formID;
    frm = FrmInitForm(formId);
    FrmSetActiveForm(frm);
    FrmSetEventHandler(frm, MainViewHandleEvent);
    handled = true;
  }
  return handled;
}

/* ----------------------------------------------------- */
/* StartApplication                                      */
/*   Initialisation etc.                                 */
/* Parms                                                 */
/*   None                                                */
/* Returns                                               */
/*   Nowt                                                */
/* ----------------------------------------------------- */

static Boolean StartApplication(void)
{
  return false;
}

/* ----------------------------------------------------- */
/* StopApplication                                       */
/*   Like my Mum used to say "Tidy up after yourself".   */
/* Parms                                                 */
/*   None                                                */
/* Returns                                               */
/*   Nowt                                                */
/* ----------------------------------------------------- */

static void StopApplication(void)
{
  FrmCloseAllForms ();
}

/* ----------------------------------------------------- */
/* EventLoop                                             */
/*   This function gets events from the Event Manager,   */
/*   passes them to the system, menu and application     */
/*   event handlers before passing them onto our own     */
/*   processing.                                         */
/* Parms                                                 */
/*   None                                                */
/* Returns                                               */
/*   Nowt                                                */
/* ----------------------------------------------------- */

static void EventLoop(void)
{
  EventType event;
  Word error;
  do
  {
    EvtGetEvent(&event, sysTicksPerSecond);
    if (! SysHandleEvent(&event))
      if (! MenuHandleEvent(NULL, &event, &error))
        if (! ApplicationHandleEvent(&event))
          FrmDispatchEvent(&event);
  }
  while (event.eType != appStopEvent);
}


/* ----------------------------------------------------- */
/* PilotMain                                             */
/*   Equivalent of the normal C Main(), it starts the    */
/*   execution of the application.                       */
/* Parms                                                 */
/*   cmd - command specifying how to launch the app      */
/*   cmdPBP - parameter block for the command            */
/*   launchFlags - flags used to config the launch       */
/* Returns                                               */
/*   An applicable error code                            */
/* ----------------------------------------------------- */

DWord PilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
Int err;

    if (cmd == sysAppLaunchCmdNormalLaunch)
    {
      err=StartApplication();
      if(err)
        return(err);
      FrmGotoForm(MainForm);
      EventLoop();
      StopApplication();
    }
    return 0;
}
